@@ -166,6 +166,7 @@ mailstream_low * mailstream_low_new(void * data,
   s->driver = driver;
   s->privacy = 1;
 	s->identifier = NULL;
+	s->timeout = 0;
   
   return s;
 }
@@ -186,6 +187,13 @@ int mailstream_low_get_fd(mailstream_low * s)
   return s->driver->mailstream_get_fd(s);
 }
 
+struct mailstream_cancel * mailstream_low_get_cancel(mailstream_low * s)
+{
+  if (s == NULL)
+    return NULL;
+  return s->driver->mailstream_get_cancel(s);
+}
+
 void mailstream_low_free(mailstream_low * s)
 {
 	free(s->identifier);
@@ -210,12 +218,18 @@ ssize_t mailstream_low_read(mailstream_low * s, void * buf, size_t count)
   }
 #endif
   
+  if (r < 0) {
+    STREAM_LOG_ERROR(s, 4, buf, 0);
+  }
+  
   return r;
 }
 
 ssize_t mailstream_low_write(mailstream_low * s,
     const void * buf, size_t count)
 {
+  ssize_t r;
+  
   if (s == NULL)
     return -1;
 
@@ -231,7 +245,13 @@ ssize_t mailstream_low_write(mailstream_low * s,
   STREAM_LOG(s, 1, ">>>>>>> end send >>>>>>\n");
 #endif
 
-  return s->driver->mailstream_write(s, buf, count);
+  r = s->driver->mailstream_write(s, buf, count);
+  
+  if (r < 0) {
+    STREAM_LOG_ERROR(s, 4 | 1, buf, 0);
+  }
+  
+  return r;
 }
 
 void mailstream_low_cancel(mailstream_low * s)
@@ -273,3 +293,15 @@ const char * mailstream_low_get_identifier(mailstream_low * s)
 {
 	return s->identifier;
 }
+
+void mailstream_low_set_timeout(mailstream_low * s,
+  time_t timeout)
+{
+	s->timeout = timeout;
+}
+
+time_t mailstream_low_get_timeout(mailstream_low * s)
+{
+	return s->timeout;
+}
+
