@@ -42,6 +42,8 @@
 #include "mailimap_sender.h"
 #include "mailimap_extension.h"
 #include "mail.h"
+#include "condstore.h"
+#include "condstore_private.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -346,25 +348,21 @@ static void mailbox_data_store(mailimap * session,
     break;
 	
   case MAILIMAP_MAILBOX_DATA_EXISTS:
-    if (session->imap_selection_info)
+    if (session->imap_selection_info) {
       session->imap_selection_info->sel_exists = mb_data->mbd_data.mbd_exists;
+      session->imap_selection_info->sel_has_exists = 1;
+    }
     break;
 
   case MAILIMAP_MAILBOX_DATA_RECENT:
-    if (session->imap_selection_info)
-      session->imap_selection_info->sel_recent =
-        mb_data->mbd_data.mbd_recent;
+    if (session->imap_selection_info) {
+      session->imap_selection_info->sel_recent = mb_data->mbd_data.mbd_recent;
+      session->imap_selection_info->sel_has_recent = 1;
+    }
     break;
   case MAILIMAP_MAILBOX_DATA_EXTENSION_DATA:
-    if (session->imap_response_info) {
-      r = clist_append(session->imap_response_info->rsp_extension_list,
-          mb_data->mbd_data.mbd_extension);
-      if (r == 0)
-	mb_data->mbd_data.mbd_extension = NULL;
-      else {
-	/* TODO must handle error case */	
-      }
-    }
+    mailimap_extension_data_store(session, &mb_data->mbd_data.mbd_extension);
+    break;
   }
 }
 
@@ -1236,6 +1234,7 @@ int mailimap_delete(mailimap * session, const char * mb)
   }
 }
 
+#if 0
 LIBETPAN_EXPORT
 int mailimap_examine(mailimap * session, const char * mb)
 {
@@ -1289,12 +1288,21 @@ int mailimap_examine(mailimap * session, const char * mb)
     return MAILIMAP_ERROR_EXAMINE;
   }
 }
+#else
+LIBETPAN_EXPORT
+int mailimap_examine(mailimap * session, const char * mb)
+{
+	uint64_t dummy;
+	return mailimap_examine_condstore_optional(session, mb, 0, &dummy);
+}
+#endif
 
 LIBETPAN_EXPORT
 int
 mailimap_fetch(mailimap * session, struct mailimap_set * set,
 	       struct mailimap_fetch_type * fetch_type, clist ** result)
 {
+#if 0
   struct mailimap_response * response;
   int r;
   int error_code;
@@ -1344,6 +1352,9 @@ mailimap_fetch(mailimap * session, struct mailimap_set * set,
     mailimap_fetch_list_free(* result);
     return MAILIMAP_ERROR_FETCH;
   }
+#else
+  return mailimap_fetch_changedsince(session, set, fetch_type, 0, result);
+#endif
 }
 
 LIBETPAN_EXPORT
@@ -1359,6 +1370,7 @@ mailimap_uid_fetch(mailimap * session,
 		   struct mailimap_set * set,
 		   struct mailimap_fetch_type * fetch_type, clist ** result)
 {
+#if 0
   struct mailimap_response * response;
   int r;
   int error_code;
@@ -1407,8 +1419,12 @@ mailimap_uid_fetch(mailimap * session,
 
   default:
     mailimap_fetch_list_free(* result);
+    * result = NULL;
     return MAILIMAP_ERROR_UID_FETCH;
   }
+#else
+  return mailimap_uid_fetch_changedsince(session, set, fetch_type, 0, result);
+#endif
 }
 
 LIBETPAN_EXPORT
@@ -1608,16 +1624,16 @@ int mailimap_authenticate(mailimap * session, const char * auth_type,
   }
   
   sasl_callback[0].id = SASL_CB_GETREALM;
-  sasl_callback[0].proc =  sasl_getrealm;
+  sasl_callback[0].proc =  (int(*)(void)) sasl_getrealm;
   sasl_callback[0].context = session;
   sasl_callback[1].id = SASL_CB_USER;
-  sasl_callback[1].proc =  sasl_getsimple;
+  sasl_callback[1].proc =  (int(*)(void)) sasl_getsimple;
   sasl_callback[1].context = session;
   sasl_callback[2].id = SASL_CB_AUTHNAME;
-  sasl_callback[2].proc =  sasl_getsimple;
+  sasl_callback[2].proc =  (int(*)(void)) sasl_getsimple;
   sasl_callback[2].context = session; 
   sasl_callback[3].id = SASL_CB_PASS;
-  sasl_callback[3].proc =  sasl_getsecret;
+  sasl_callback[3].proc =  (int(*)(void)) sasl_getsecret;
   sasl_callback[3].context = session;
   sasl_callback[4].id = SASL_CB_LIST_END;
   sasl_callback[4].proc =  NULL;
@@ -1945,6 +1961,7 @@ int
 mailimap_search(mailimap * session, const char * charset,
     struct mailimap_search_key * key, clist ** result)
 {
+#if 0
   struct mailimap_response * response;
   int r;
   int error_code;
@@ -1988,6 +2005,9 @@ mailimap_search(mailimap * session, const char * charset,
   default:
     return MAILIMAP_ERROR_SEARCH;
   }
+#else
+  return mailimap_search_modseq(session, charset, key, result, NULL);
+#endif
 }
 
 LIBETPAN_EXPORT
@@ -1995,6 +2015,7 @@ int
 mailimap_uid_search(mailimap * session, const char * charset,
     struct mailimap_search_key * key, clist ** result)
 {
+#if 0
   struct mailimap_response * response;
   int r;
   int error_code;
@@ -2038,6 +2059,9 @@ mailimap_uid_search(mailimap * session, const char * charset,
   default:
     return MAILIMAP_ERROR_UID_SEARCH;
   }
+#else
+  return mailimap_uid_search_modseq(session, charset, key, result, NULL);
+#endif
 }
 
 LIBETPAN_EXPORT
@@ -2047,6 +2071,7 @@ void mailimap_search_result_free(clist * search_result)
   clist_free(search_result);
 }
 
+#if 0
 LIBETPAN_EXPORT
 int
 mailimap_select(mailimap * session, const char * mb)
@@ -2101,6 +2126,15 @@ mailimap_select(mailimap * session, const char * mb)
     return MAILIMAP_ERROR_SELECT;
   }
 }
+#else
+LIBETPAN_EXPORT
+int
+mailimap_select(mailimap * session, const char * mb)
+{
+	uint64_t dummy;
+	return mailimap_select_condstore_optional(session, mb, 0, &dummy);
+}
+#endif
 
 LIBETPAN_EXPORT
 int
@@ -2161,6 +2195,7 @@ mailimap_store(mailimap * session,
 	       struct mailimap_set * set,
 	       struct mailimap_store_att_flags * store_att_flags)
 {
+#if 0
   struct mailimap_response * response;
   int r;
   int error_code;
@@ -2201,6 +2236,10 @@ mailimap_store(mailimap * session,
   default:
     return MAILIMAP_ERROR_STORE;
   }
+#else
+  return mailimap_store_unchangedsince_optional(session,
+  	set, 0, 0, store_att_flags);
+#endif
 }
 
 LIBETPAN_EXPORT
@@ -2209,6 +2248,7 @@ mailimap_uid_store(mailimap * session,
 		   struct mailimap_set * set,
 		   struct mailimap_store_att_flags * store_att_flags)
 {
+#if 0
   struct mailimap_response * response;
   int r;
   int error_code;
@@ -2249,6 +2289,10 @@ mailimap_uid_store(mailimap * session,
   default:
     return MAILIMAP_ERROR_UID_STORE;
   }
+#else
+  return mailimap_uid_store_unchangedsince_optional(session,
+  	set, 0, 0, store_att_flags);
+#endif
 }
 
 LIBETPAN_EXPORT
@@ -2439,7 +2483,9 @@ int mailimap_parse_response(mailimap * session,
                                              &indx, &response,
                                              session->imap_body_progress_fun,
                                              session->imap_items_progress_fun,
-                                             session->imap_progress_context);
+                                             session->imap_progress_context,
+                                             session->imap_msg_att_handler,
+                                             session->imap_msg_att_handler_context);
   }
   else {
     r = mailimap_response_parse(session->imap_stream,
@@ -2582,7 +2628,12 @@ mailimap * mailimap_new(size_t imap_progr_rate,
   f->imap_body_progress_fun = NULL;
   f->imap_items_progress_fun = NULL;
   f->imap_progress_context = NULL;
+    
+  f->imap_msg_att_handler = NULL;
+  f->imap_msg_att_handler_context = NULL;
   
+	f->imap_timeout = 0;
+
   return f;
   
  free_stream_buffer:
@@ -2619,6 +2670,18 @@ void mailimap_free(mailimap * session)
   free(session);
 }
 
+LIBETPAN_EXPORT
+void mailimap_set_timeout(mailimap * session, time_t timeout)
+{
+	session->imap_timeout = timeout;
+}
+
+LIBETPAN_EXPORT
+time_t mailimap_get_timeout(mailimap * session)
+{
+	return session->imap_timeout;
+}
+
 LIBETPAN_EXPORT
 void mailimap_set_progress_callback(mailimap * session,
                                     mailprogress_function * body_progr_fun,
@@ -2629,3 +2692,14 @@ void mailimap_set_progress_callback(mailimap * session,
   session->imap_items_progress_fun = items_progr_fun;
   session->imap_progress_context = context;
 }
+
+LIBETPAN_EXPORT
+void mailimap_set_msg_att_handler(mailimap * session,
+                                  mailimap_msg_att_handler * handler,
+                                  void * context)
+{
+    session->imap_msg_att_handler = handler;
+    session->imap_msg_att_handler_context = context;
+}
+
+
