@@ -54,7 +54,16 @@ extern "C" {
 #include <libetpan/idle.h>
 #include <libetpan/quota.h>
 #include <libetpan/namespace.h>
+#include <libetpan/mailimap_id.h>
+#include <libetpan/enable.h>
 #include <libetpan/xlist.h>
+#include <libetpan/xgmlabels.h>
+#include <libetpan/xgmmsgid.h>
+#include <libetpan/xgmthrid.h>
+#include <libetpan/condstore.h>
+#include <libetpan/qresync.h>
+#include <libetpan/mailimap_sort.h>
+#include <libetpan/mailimap_compress.h>
 
 /*
   mailimap_connect()
@@ -608,6 +617,8 @@ int mailimap_unsubscribe(mailimap * session, const char * mb);
 
    This function starts change the mode of the connection to
    switch to SSL connection.
+   It won't change the stream connection to SSL rightway.
+   See mailimap_socket_starttls() will switch the mailstream too.
 
    @param session   IMAP session
 
@@ -661,6 +672,17 @@ void mailimap_set_progress_callback(mailimap * session,
                                     mailprogress_function * items_progr_fun,
                                     void * context);
 
+LIBETPAN_EXPORT
+void mailimap_set_msg_att_handler(mailimap * session,
+                                  mailimap_msg_att_handler * handler,
+                                  void * context);
+
+LIBETPAN_EXPORT
+void mailimap_set_timeout(mailimap * session, time_t timeout);;
+
+LIBETPAN_EXPORT
+time_t mailimap_get_timeout(mailimap * session);
+
 #ifdef __cplusplus
 }
 #endif
