@@ -225,6 +225,10 @@ mailpop3 * mailpop3_new(size_t progr_rate, progress_function * progr_fun)
 #ifdef USE_SASL
   f->pop3_sasl.sasl_conn = NULL;
 #endif
+
+	f->pop3_timeout = 0;
+	f->pop3_progress_fun = NULL;
+	f->pop3_progress_context = NULL;
   
   return f;
 
@@ -383,20 +387,26 @@ int mailpop3_quit(mailpop3 * f)
     res = MAILPOP3_ERROR_STREAM;
     goto close;
   }
-  parse_response(f, response);
+  if (parse_response(f, response) == RESPONSE_OK) {
+		res = MAILPOP3_NO_ERROR;
+	}
+	else {
+		res = MAILPOP3_ERROR_QUIT_FAILED;
+	}
 
   res = MAILPOP3_NO_ERROR;
 
  close:
-  if (f->pop3_state != POP3_STATE_DISCONNECTED)
+  if (f->pop3_stream != NULL) {
     mailstream_close(f->pop3_stream);
+    f->pop3_stream = NULL;
+  }
 
   if (f->pop3_timestamp != NULL) {
     free(f->pop3_timestamp);
     f->pop3_timestamp = NULL;
   }
 
-  f->pop3_stream = NULL;
   if (f->pop3_msg_tab != NULL) {
     mailpop3_msg_info_tab_free(f->pop3_msg_tab);
     f->pop3_msg_tab = NULL;
@@ -1032,6 +1042,8 @@ static int parse_response(mailpop3 * f, char * response)
       f->pop3_response = f->pop3_response_buffer->str;
     else
       f->pop3_response = NULL;
+
+	  return RESPONSE_ERR;
   }
 
   f->pop3_response = NULL;
@@ -1275,7 +1287,7 @@ static char * read_multiline(mailpop3 * f, size_t size,
 {
   return mailstream_read_multiline(f->pop3_stream, size,
 				   f->pop3_stream_buffer, multiline_buffer,
-				   f->pop3_progr_rate, f->pop3_progr_fun);
+				   f->pop3_progr_rate, f->pop3_progr_fun, f->pop3_progress_fun, f->pop3_progress_context);
 }
 
 static int send_command(mailpop3 * f, char * command)
@@ -1386,16 +1398,16 @@ int mailpop3_auth(mailpop3 * f, const char * auth_type,
   unsigned int max_encoded;
   
   sasl_callback[0].id = SASL_CB_GETREALM;
-  sasl_callback[0].proc =  sasl_getrealm;
+  sasl_callback[0].proc =  (int(*)(void)) sasl_getrealm;
   sasl_callback[0].context = f;
   sasl_callback[1].id = SASL_CB_USER;
-  sasl_callback[1].proc =  sasl_getsimple;
+  sasl_callback[1].proc =  (int(*)(void)) sasl_getsimple;
   sasl_callback[1].context = f;
   sasl_callback[2].id = SASL_CB_AUTHNAME;
-  sasl_callback[2].proc =  sasl_getsimple;
+  sasl_callback[2].proc =  (int(*)(void)) sasl_getsimple;
   sasl_callback[2].context = f; 
   sasl_callback[3].id = SASL_CB_PASS;
-  sasl_callback[3].proc =  sasl_getsecret;
+  sasl_callback[3].proc =  (int(*)(void)) sasl_getsecret;
   sasl_callback[3].context = f;
   sasl_callback[4].id = SASL_CB_LIST_END;
   sasl_callback[4].proc =  NULL;
@@ -1561,3 +1573,20 @@ int mailpop3_auth(mailpop3 * f, const char * auth_type,
   return MAILPOP3_ERROR_BAD_USER;
 #endif
 }
+
+void mailpop3_set_timeout(mailpop3 * f, time_t timeout)
+{
+	f->pop3_timeout = timeout;
+}
+
+time_t mailpop3_get_timeout(mailpop3 * f)
+{
+	return f->pop3_timeout;
+}
+
+void mailpop3_set_progress_callback(mailpop3 * f, mailprogress_function * progr_fun, void * context)
+{
+	f->pop3_progress_fun = progr_fun;
+	f->pop3_progress_context = context;
+}
+
