@@ -30,7 +30,7 @@
  */
 
 /*
- * $Id: mailpop3.h,v 1.17 2006/12/13 18:31:33 hoa Exp $
+ * $Id: mailpop3.h,v 1.19 2010/04/05 12:56:44 hoa Exp $
  */
 
 #ifndef MAILPOP3_H
@@ -57,6 +57,15 @@ mailpop3 * mailpop3_new(size_t pop3_progr_rate,
 LIBETPAN_EXPORT
 void mailpop3_free(mailpop3 * f);
 
+LIBETPAN_EXPORT
+void mailpop3_set_timeout(mailpop3 * f, time_t timeout);
+
+LIBETPAN_EXPORT
+time_t mailpop3_get_timeout(mailpop3 * f);
+
+LIBETPAN_EXPORT
+void mailpop3_set_progress_callback(mailpop3 * f, mailprogress_function * progr_fun, void * context);
+
 LIBETPAN_EXPORT
 int mailpop3_connect(mailpop3 * f, mailstream * s);
 
@@ -73,19 +82,19 @@ LIBETPAN_EXPORT
 int mailpop3_pass(mailpop3 * f, const char * password);
 
 LIBETPAN_EXPORT
-void mailpop3_list(mailpop3 * f, carray ** result);
+int mailpop3_list(mailpop3 * f, carray ** result);
 
 LIBETPAN_EXPORT
-int mailpop3_retr(mailpop3 * f, unsigned int index, char ** result,
+int mailpop3_retr(mailpop3 * f, unsigned int indx, char ** result,
 		  size_t * result_len);
 
 LIBETPAN_EXPORT
-int mailpop3_top(mailpop3 * f, unsigned int index,
+int mailpop3_top(mailpop3 * f, unsigned int indx,
     unsigned int count, char ** result,
     size_t * result_len);
 
 LIBETPAN_EXPORT
-int mailpop3_dele(mailpop3 * f, unsigned int index);
+int mailpop3_dele(mailpop3 * f, unsigned int indx);
 
 LIBETPAN_EXPORT
 int mailpop3_noop(mailpop3 * f);
@@ -100,7 +109,7 @@ LIBETPAN_EXPORT
 void mailpop3_retr_free(char * str);
 
 LIBETPAN_EXPORT
-int mailpop3_get_msg_info(mailpop3 * f, unsigned int index,
+int mailpop3_get_msg_info(mailpop3 * f, unsigned int indx,
 			   struct mailpop3_msg_info ** result);
 
 LIBETPAN_EXPORT
