@@ -44,8 +44,19 @@
 #include "base64.h"
 #include "mail.h"
 
+#ifdef HAVE_SYS_SOCKET_H
+#	include <sys/socket.h>
+#endif
+
+#ifdef HAVE_STRING_H
+#  include <string.h>
+#endif
+
 #ifdef WIN32
 #	include "win_etpan.h"
+#else
+#	include <sys/types.h>
+#   include <netdb.h>
 #endif
 
 #ifdef HAVE_NETINET_IN_H
@@ -104,6 +115,7 @@ mailsmtp * mailsmtp_new(size_t progr_rate,
   session->progr_fun = progr_fun;
 
   session->response = NULL;
+	session->response_code = 0;
 
   session->line_buffer = mmap_string_new("");
   if (session->line_buffer == NULL)
@@ -124,6 +136,8 @@ mailsmtp * mailsmtp_new(size_t progr_rate,
   session->smtp_progress_fun = NULL;
   session->smtp_progress_context = NULL;
 
+	session->smtp_timeout = 0;
+
   return session;
 
  free_line_buffer:
@@ -156,6 +170,8 @@ static int send_command_private(mailsmtp * f, char * command, int can_be_publish
 
 static int read_response(mailsmtp * session);
 
+static int get_hostname(mailsmtp * session, int useip, char * buf, int len);
+
 /* smtp operations */
 
 int mailsmtp_connect(mailsmtp * session, mailstream * s)
@@ -185,21 +201,31 @@ int mailsmtp_connect(mailsmtp * session, mailstream * s)
 
 #define SMTP_STRING_SIZE 513
 
+static int send_quit(mailsmtp * session)
+{
+    char command[SMTP_STRING_SIZE];
+    int r;
+    
+    snprintf(command, SMTP_STRING_SIZE, "QUIT\r\n");
+    r = send_command(session, command);
+    if (r == -1) {
+        return MAILSMTP_ERROR_STREAM;
+    }
+    
+    return MAILSMTP_NO_ERROR;
+}
+
 int mailsmtp_quit(mailsmtp * session)
 {
-  char command[SMTP_STRING_SIZE];
   int r;
   int res;
   
-  snprintf(command, SMTP_STRING_SIZE, "QUIT\r\n");
-  r = send_command(session, command);
-  if (r == -1) {
-    res = MAILSMTP_ERROR_STREAM;
-    goto close;
-  }
-  r = read_response(session);
-  if (r == 0) {
-    res = MAILSMTP_ERROR_STREAM;
+  if (session->stream == NULL)
+    return MAILSMTP_NO_ERROR;
+  
+  r = send_quit(session);
+  if (r != MAILSMTP_NO_ERROR) {
+    res = r;
     goto close;
   }
   
@@ -213,18 +239,61 @@ int mailsmtp_quit(mailsmtp * session)
 }
 
 
-
 #define HOSTNAME_SIZE 256
 
+static int get_hostname(mailsmtp * session, int useip, char * buf, int len)
+{
+  int r;
+  char hostname[HOSTNAME_SIZE];
+  struct sockaddr addr;
+  socklen_t addr_len = sizeof(addr);
+  int socket = -1;
+
+  if (!useip) {
+    r = gethostname(hostname, HOSTNAME_SIZE);
+    if (r != 0)
+      return MAILSMTP_ERROR_HOSTNAME;
+
+    if (snprintf(buf, len, "%s", hostname) >= len)
+      return MAILSMTP_ERROR_HOSTNAME;
+  } else {
+    socket = mailstream_low_get_fd(mailstream_get_low(session->stream));
+    if (socket < 0)
+      return MAILSMTP_ERROR_HOSTNAME;
+
+    r = getsockname(socket, &addr, &addr_len );
+    if (r != 0)
+      return MAILSMTP_ERROR_HOSTNAME;
+
+#if (defined __linux__ || defined WIN32)
+    r = getnameinfo(&addr, sizeof(addr), hostname, HOSTNAME_SIZE, NULL, 0, NI_NUMERICHOST);
+#else
+    r = getnameinfo(&addr, addr.sa_len, hostname, HOSTNAME_SIZE, NULL, 0, NI_NUMERICHOST);
+#endif
+    if (r != 0)
+      return MAILSMTP_ERROR_HOSTNAME;
+
+    if (snprintf(buf, len, "[%s]", hostname) >= len)
+      return MAILSMTP_ERROR_HOSTNAME;
+  }
+   return MAILSMTP_NO_ERROR;
+}
+
+
 int mailsmtp_helo(mailsmtp * session)
+{
+  return mailsmtp_helo_with_ip(session, 0);
+}
+
+int mailsmtp_helo_with_ip(mailsmtp * session, int useip)
 {
   int r;
   char hostname[HOSTNAME_SIZE];
   char command[SMTP_STRING_SIZE];
 
-  r = gethostname(hostname, HOSTNAME_SIZE);
-  if (r < 0)
-    return MAILSMTP_ERROR_HOSTNAME;
+  r = get_hostname(session, useip, hostname, HOSTNAME_SIZE);
+  if (r != MAILSMTP_NO_ERROR)
+    return r;
 
   snprintf(command, SMTP_STRING_SIZE, "HELO %s\r\n", hostname);
   r = send_command(session, command);
@@ -365,6 +434,47 @@ int mailsmtp_data_message(mailsmtp * session,
   }
 }
 
+int mailsmtp_data_message_quit(mailsmtp * session,
+                               const char * message,
+                               size_t size)
+{
+    int r;
+    
+    r = send_data(session, message, size);
+    if (r == -1)
+        return MAILSMTP_ERROR_STREAM;
+    
+    r = send_quit(session);
+    
+    r = read_response(session);
+    
+    mailstream_close(session->stream);
+    session->stream = NULL;
+    
+    switch(r) {
+        case 250:
+            return MAILSMTP_NO_ERROR;
+            
+        case 552:
+            return MAILSMTP_ERROR_EXCEED_STORAGE_ALLOCATION;
+            
+        case 554:
+            return MAILSMTP_ERROR_TRANSACTION_FAILED;
+            
+        case 451:
+            return MAILSMTP_ERROR_IN_PROCESSING;
+            
+        case 452:
+            return MAILSMTP_ERROR_INSUFFICIENT_SYSTEM_STORAGE;
+            
+        case 0:
+            return MAILSMTP_ERROR_STREAM;
+            
+        default:
+            return MAILSMTP_ERROR_UNEXPECTED_CODE;
+    }
+}
+
 /* esmtp operations */
 
 
@@ -419,6 +529,9 @@ int mailesmtp_parse_ehlo(mailsmtp * session)
       }
       /* TODO: grab optionnal max size */
     }
+    else if (!strncasecmp(response, "PIPELINING", 10) && isdelim(response[10])) {
+      session->esmtp |= MAILSMTP_ESMTP_PIPELINING;
+    }
     else if (!strncasecmp(response, "AUTH ", 5)) {
       response += 5;       /* remove "AUTH " */
       while (response[0] != '\n' && response[0] != '\0') {
@@ -500,14 +613,19 @@ int mailesmtp_parse_ehlo(mailsmtp * session)
 
 
 int mailesmtp_ehlo(mailsmtp * session)
+{
+  return mailesmtp_ehlo_with_ip(session, 0);
+}
+
+int mailesmtp_ehlo_with_ip(mailsmtp * session, int useip)
 {
   int r;
   char hostname[HOSTNAME_SIZE];
   char command[SMTP_STRING_SIZE];
 
-  r = gethostname(hostname, HOSTNAME_SIZE);
-  if (r != 0)
-    return MAILSMTP_ERROR_HOSTNAME;
+  r = get_hostname(session, useip, hostname, HOSTNAME_SIZE);
+  if (r != MAILSMTP_NO_ERROR)
+    return r;
 
   snprintf(command, SMTP_STRING_SIZE, "EHLO %s\r\n", hostname);
   r = send_command(session, command);
@@ -835,7 +953,8 @@ int mailsmtp_auth(mailsmtp * session, const char * user, const char * pass)
 
 /* TODO: add mailesmtp_etrn, mailssmtp_expn */
 
-int mailesmtp_starttls(mailsmtp * session) {
+int mailesmtp_starttls(mailsmtp * session)
+{
   int r;
 
   if (!(session->esmtp & MAILSMTP_ESMTP_STARTTLS))
@@ -904,7 +1023,8 @@ static int read_response(mailsmtp * session)
   while ((code & SMTP_STATUS_CONTINUE) != 0);
 
   session->response = session->response_buffer->str;
-
+	session->response_code = code;
+    
   return code;
 }
 
@@ -993,6 +1113,8 @@ const char * mailsmtp_strerror(int errnum)
     return "TLS not available on server for temporary reason";
   case MAILSMTP_ERROR_STARTTLS_NOT_SUPPORTED:
     return "TLS not supported by server";
+  case MAILSMTP_ERROR_AUTH_LOGIN:
+  	return "Login failed";
   default:
     return "Unknown error code";
   }
@@ -1084,16 +1206,16 @@ int mailesmtp_auth_sasl(mailsmtp * session, const char * auth_type,
   unsigned int max_encoded;
   
   sasl_callback[0].id = SASL_CB_GETREALM;
-  sasl_callback[0].proc =  sasl_getrealm;
+  sasl_callback[0].proc =  (int(*)(void)) sasl_getrealm;
   sasl_callback[0].context = session;
   sasl_callback[1].id = SASL_CB_USER;
-  sasl_callback[1].proc =  sasl_getsimple;
+  sasl_callback[1].proc =  (int(*)(void)) sasl_getsimple;
   sasl_callback[1].context = session;
   sasl_callback[2].id = SASL_CB_AUTHNAME;
-  sasl_callback[2].proc =  sasl_getsimple;
+  sasl_callback[2].proc =  (int(*)(void)) sasl_getsimple;
   sasl_callback[2].context = session; 
   sasl_callback[3].id = SASL_CB_PASS;
-  sasl_callback[3].proc =  sasl_getsecret;
+  sasl_callback[3].proc =  (int(*)(void)) sasl_getsecret;
   sasl_callback[3].context = session;
   sasl_callback[4].id = SASL_CB_LIST_END;
   sasl_callback[4].proc =  NULL;
@@ -1319,3 +1441,14 @@ void mailsmtp_set_progress_callback(mailsmtp * session,
   session->smtp_progress_fun = progr_fun;
   session->smtp_progress_context = context;
 }
+
+void mailsmtp_set_timeout(mailsmtp * session, time_t timeout)
+{
+	session->smtp_timeout = timeout;
+}
+
+time_t mailsmtp_get_timeout(mailsmtp * session)
+{
+	return session->smtp_timeout;
+}
+
